#include <IRremote.h> // Here I include all of the libraries needed for the code including one for an ir remote modification for the future
#include <SPI.h> // Previous
#include <MFRC522.h> // This is the library for the RFID Sensor
#include <LiquidCrystal_I2C.h> // This library is for the LCD Screen
#include <StepperAK.h> // This is the library for the Stepper Motors
#include <Wire.h> // This is just a general library for some basic functionality codes
 
#define SS_PIN 9 // This is a specification for the RFID Sensor for communication.
#define RST_PIN 8 // This is a specification for the RFID Sensor to Reset after having received a new message.

#define gearratio 64 // This is a specification for the Stepper Motors

const int stepsPerRevolution = 2048;
const int SENSOR_PIN = A0; // Here I define the pin that is going to be connected to the first touch sensor
const int SENSOR_PIN_2 = A1; // Same as above all the way through sensor pin 8
const int SENSOR_PIN_3 = A2;
const int SENSOR_PIN_4 = A8;
const int SENSOR_PIN_5 = A9;
const int SENSOR_PIN_6 = A10;
const int SENSOR_PIN_7 = A0;
const int SENSOR_PIN_8 = A10;
int lastState_1 = LOW; // Here I create a variable representing the touch sensors and specify the state of it as LOW      
int lastState_2 = LOW;
int lastState_3 = LOW;
int lastState_4 = LOW;
int lastState_5 = LOW;
int lastState_6 = LOW;
int lastState_7 = LOW;
int lastState_8 = LOW;
int currentState_1; // Here I create variables to mark the new state of each touch sensor        
int currentState_2;
int currentState_3;
int currentState_4;
int currentState_5;
int currentState_6;
int currentState_7;
int currentState_8;

int timer; // This is a variable that I am going to be using as a timer

int RECV_PIN = A15; // This is the pin that could be used for the IR SENSOR incorporation

MFRC522 mfrc522(SS_PIN, RST_PIN); // This is another specification for the RFID SENSOR
IRrecv irrecv(RECV_PIN); // This would be another command needed for the successful integration of the IR SENSOR
decode_results results; // Same as previous
Stepper myStepper(stepsPerRevolution, 24,26,28,30); // Defines the pins being used for the first stepper
Stepper thirdStepper(stepsPerRevolution, 39,41,43,45); // Defines the pins being used for the third stepper
Stepper secondStepper(stepsPerRevolution, 2,3,4,5); // Defines the pins being used for the second stepper
LiquidCrystal_I2C lcd(0x27,16,2); // This is a descriptor for the LCD Display to specify which type of LCD I am using
 
void setup() 
{
  Serial.begin(9600); // Serial communication begins
  
  myStepper.setSpeed(0.15*gearratio); // The speed for the Stepper Motors is specified
  secondStepper.setSpeed(0.15*gearratio);
  thirdStepper.setSpeed(0.15*gearratio);


  pinMode(SENSOR_PIN, INPUT); // Here I define the touch sensors as inputs
  pinMode(SENSOR_PIN_2, INPUT);
  pinMode(SENSOR_PIN_3, INPUT);
  pinMode(SENSOR_PIN_4, INPUT);
  pinMode(SENSOR_PIN_5, INPUT);
  pinMode(SENSOR_PIN_6, INPUT);
  pinMode(SENSOR_PIN_7, INPUT);
  pinMode(SENSOR_PIN_8, INPUT);
  
  
  SPI.begin();      // Initiate  SPI bus
  mfrc522.PCD_Init();   // Initiate MFRC522
  Serial.println("Approximate your card to the reader..."); // General communication with the serial port
  Serial.println(); // Moves to a new line on Serial Port Monitor
  
  lcd.init();  // Begins the setup for the LCD                   
  lcd.backlight(); // Continues setup
  lcd.clear(); // Clears the LCD display
  lcd.print("Locked!!!!"); // Writes the word in "" on the LCD display
  
  irrecv.enableIRIn(); // Final component for the setup for the IR Sensor
}
void loop() // Begins the main loop
{
  // Look for new cards
  if ( ! mfrc522.PICC_IsNewCardPresent()) 
  {
    return;
  }
  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial()) 
  {
    return;
  }
  //Show UID on serial monitor
  Serial.print("UID tag :");
  String content= "";
  byte letter;
  for (byte i = 0; i < mfrc522.uid.size; i++) 
  {
     Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
     Serial.print(mfrc522.uid.uidByte[i], HEX);
     content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
     content.concat(String(mfrc522.uid.uidByte[i], HEX));
  }
  Serial.println();
  Serial.print("Message : ");
  content.toUpperCase();
  if (content.substring(1) == "66 95 B1 F8") // Begins an if loop with the condition of the RFID tag/card with the UID tag as specified
    {
    Serial.println("Authorized access"); // General communication with Serial
    Serial.println();
    delay(1500);
    
    lcd.clear(); 
    lcd.print("RFID Scanned"); // Specifies that the RFID has been scanned
    
    currentState_1 = digitalRead(SENSOR_PIN); // Establishes the state of the touch sensor as LOW or not being touched by identifying if it has been touched or not.
    currentState_2 = digitalRead(SENSOR_PIN_2);
    currentState_3 = digitalRead(SENSOR_PIN_3);
    currentState_4 = digitalRead(SENSOR_PIN_4);
    currentState_5 = digitalRead(SENSOR_PIN_5);
    currentState_6 = digitalRead(SENSOR_PIN_6);
    currentState_7 = digitalRead(SENSOR_PIN_7);
    currentState_8 = digitalRead(SENSOR_PIN_8);

    delay(3000); // Creates a delay of 3 seconds, allowing for you to do the next thing before the code reads the next step

    if(lastState_1 == LOW && currentState_1 == HIGH) // Compares the previous state of the touch sensor and the current sensor to determine if it has been touched
      {
        
        Serial.println("First sensor is touched "); // General serial communication
        lcd.clear();
        lcd.print("Unlocking: 12.5%"); // Lets the user know how much progress they've made with unlocking the box
        delay(2000); // Same use as before
        currentState_2 = digitalRead(SENSOR_PIN_2); // Checks the state of the next touch sensor to figure out if it has been touched or not in the next step.
        if(lastState_2 == LOW && currentState_2 == HIGH) // Process repeats for all sensors in the form of stacked if loops
          {
            Serial.println("Second sensor is touched ");
            lcd.clear();
            lcd.print("Unlocking: 25%");
            delay(2000);
            currentState_3 = digitalRead(SENSOR_PIN_3);
            if(lastState_3 == LOW && currentState_3 == HIGH)
              {
                Serial.println("Third sensor is touched ");
                lcd.clear();
                lcd.print("Unlocking: 37.5%");
                delay(2000);
                currentState_4 = digitalRead(SENSOR_PIN_4);
                if(lastState_4 == LOW && currentState_4 == HIGH)
                  {
                    Serial.println("Fourth sensor is touched ");
                    lcd.clear();
                    lcd.print("Unlocking: 50%");
                    delay(2000);
                    currentState_5 = digitalRead(SENSOR_PIN_5);
                    if(lastState_5 == LOW && currentState_5 == HIGH)
                      {
                        Serial.println("Fifth sensor is touched ");
                        lcd.clear();
                        lcd.print("Unlocking: 62.5%");
                        delay(2000);
                        currentState_6 = digitalRead(SENSOR_PIN_6);
                        if(lastState_6 == LOW && currentState_6 == HIGH)
                          {
                            Serial.println("Sixth sensor is touched ");
                            lcd.clear();
                            lcd.print("Unlocking: 75%");
                            delay(2000);
                            currentState_7 = digitalRead(SENSOR_PIN_7);
                            if(lastState_7 == LOW && currentState_7 == HIGH)
                              {
                                Serial.println("Seventh sensor is touched ");
                                lcd.clear();
                                lcd.print("Unlocking: 87.5%");
                                delay(2000);
                                currentState_8 = digitalRead(SENSOR_PIN_8);
                                if(lastState_8 == LOW && currentState_8 == HIGH)
                                  {
                                    Serial.println("Eighth sensor is touched ");
                                    lcd.clear();
                                    lcd.print("Unlocking!!!!"); // Begins the sequence of unlocking the box and displays it on the LCD
                                    delay(2000); // Same use as before
                                    Serial.println("clockwise");
                                    myStepper.step(-2*stepsPerRevolution); // Rotates the Steppers that have the locking mechanism
                                    thirdStepper.step(2*stepsPerRevolution);
                                    secondStepper.step(4*stepsPerRevolution); // Rotates the string locking the box in place, therefore releasing the box
                                    lcd.clear();
                                    lcd.print("Unlocked!!!"); // Lets user know the main drawer is ready to be pulled
                                    delay(2000);
                  
                                    for (int i = 15; i > 0; i--) // Create a timer that counts from 15 downwards marking seconds that you have until the box locks again
                                      {
                                        timer = i;
                                        lcd.clear();
                                        lcd.print(timer);
                                        delay(1000);
                                      }
                                    lcd.clear();
                                    lcd.print("Locking!!!"); // Begins the locking sequence and display it on the LCD
                                    Serial.println("counterclockwise"); 
                                    secondStepper.step(-4*stepsPerRevolution); // Rotates the string therefore pulling in the main drawer
                                    thirdStepper.step(-3*stepsPerRevolution); // Rotates the locking pins to keep drawer locked
                                    myStepper.step(3*stepsPerRevolution);
                                    lcd.clear();
                                    lcd.print("Locked!!!!"); // Displays that the box is again locked.
                                  }
                              }
                          }
                      }
                  }
              }
          }
      }
    
    lastState_1 = currentState_1; // Defines the new state of the touch sensors
    lastState_2 = currentState_2;
    lastState_3 = currentState_3;
    lastState_4 = currentState_4;
    lastState_5 = currentState_5;
    lastState_6 = currentState_6;
    lastState_7 = currentState_7;
    lastState_8 = currentState_8;
    }

  else if (content.substring(1) == "9B 8D 99 22") // The same loop as previous except with a different tag/card
  {
    Serial.println("Authorized access");
    Serial.println();
    delay(1500);
    
    lcd.clear();
    lcd.print("RFID Scanned");
    
    currentState_1 = digitalRead(SENSOR_PIN);
    currentState_2 = digitalRead(SENSOR_PIN_2);
    currentState_3 = digitalRead(SENSOR_PIN_3);
    currentState_4 = digitalRead(SENSOR_PIN_4);
    currentState_5 = digitalRead(SENSOR_PIN_5);
    currentState_6 = digitalRead(SENSOR_PIN_6);
    currentState_7 = digitalRead(SENSOR_PIN_7);
    currentState_8 = digitalRead(SENSOR_PIN_8);

    delay(3000);
          
    if(lastState_1 == LOW && currentState_1 == HIGH)
      {
        
        Serial.println("First sensor is touched ");
        lcd.clear();
        lcd.print("Unlocking: 12.5%");
        delay(2000);
        currentState_2 = digitalRead(SENSOR_PIN_2);
        if(lastState_2 == LOW && currentState_2 == HIGH)
          {
            Serial.println("Second sensor is touched ");
            lcd.clear();
            lcd.print("Unlocking: 25%");
            delay(2000);
            currentState_3 = digitalRead(SENSOR_PIN_3);
            if(lastState_3 == LOW && currentState_3 == HIGH)
              {
                Serial.println("Third sensor is touched ");
                lcd.clear();
                lcd.print("Unlocking: 37.5%");
                delay(2000);
                currentState_4 = digitalRead(SENSOR_PIN_4);
                if(lastState_4 == LOW && currentState_4 == HIGH)
                  {
                    Serial.println("Fourth sensor is touched ");
                    lcd.clear();
                    lcd.print("Unlocking: 50%");
                    delay(2000);
                    currentState_5 = digitalRead(SENSOR_PIN_5);
                    if(lastState_5 == LOW && currentState_5 == HIGH)
                      {
                        Serial.println("Fifth sensor is touched ");
                        lcd.clear();
                        lcd.print("Unlocking: 62.5%");
                        delay(2000);
                        currentState_6 = digitalRead(SENSOR_PIN_6);
                        if(lastState_6 == LOW && currentState_6 == HIGH)
                          {
                            Serial.println("Sixth sensor is touched ");
                            lcd.clear();
                            lcd.print("Unlocking: 75%");
                            delay(2000);
                            currentState_7 = digitalRead(SENSOR_PIN_7);
                            if(lastState_7 == LOW && currentState_7 == HIGH)
                              {
                                Serial.println("Seventh sensor is touched ");
                                lcd.clear();
                                lcd.print("Unlocking: 87.5%");
                                delay(2000);
                                currentState_8 = digitalRead(SENSOR_PIN_8);
                                if(lastState_8 == LOW && currentState_8 == HIGH)
                                  {
                                    Serial.println("Eighth sensor is touched ");
                                    lcd.clear();
                                    lcd.print("Unlocking!!!!");
                                    delay(2000);
                                    Serial.println("clockwise");
                                    myStepper.step(-2*stepsPerRevolution);
                                    thirdStepper.step(2*stepsPerRevolution);
                                    secondStepper.step(4*stepsPerRevolution);
                                    lcd.clear();
                                    lcd.print("Unlocked!!!");
                                    delay(2000);
                  
                                    for (int i = 15; i > 0; i--)
                                      {
                                        timer = i;
                                        lcd.clear();
                                        lcd.print(timer);
                                        delay(1000);
                                      }
                                    lcd.clear();
                                    lcd.print("Locking!!!");
                                    Serial.println("counterclockwise");
                                    secondStepper.step(-4*stepsPerRevolution);
                                    thirdStepper.step(-3*stepsPerRevolution);
                                    myStepper.step(3*stepsPerRevolution);
                                    lcd.clear();
                                    lcd.print("Locked!!!!");
                                  }
                              }
                          }
                      }
                  }
              }
          }
      }
    
    lastState_1 = currentState_1;
    lastState_2 = currentState_2;
    lastState_3 = currentState_3;
    lastState_4 = currentState_4;
    lastState_5 = currentState_5;
    lastState_6 = currentState_6;
    lastState_7 = currentState_7;
    lastState_8 = currentState_8;
  }

 
 else   {
    Serial.println(" Access denied"); // If anything else is scanned on the RFID, the serial will display Access Denied
    Serial.println();
    delay(1500);
    
    lastState_1 = currentState_1;
    lastState_2 = currentState_2;
    lastState_3 = currentState_3;
    lastState_4 = currentState_4;
    lastState_5 = currentState_5;
    lastState_6 = currentState_6;
    lastState_7 = currentState_7;
    lastState_8 = currentState_8;
  }
  lcd.clear();
  lcd.print("Locked!!!!");
  
} 
